@import url(html4css1.css);

html {
    color: black;
    background-color: white;
}

body {
    margin-left: 10ex;
    margin-top: 5ex;
    padding-left: 1ex;
    border-left: 1px solid #006;

    width: 75ex;
    background-color: white;
}

h1 {
    border-bottom: 2px solid #006;
}

p {
    text-align: justify;
}

dt {
    font-weight: bold;
}
